@@ -51,15 +51,35 @@ def get_defining_component(pe_pe):
         return None
     
     if pe_pe.__class__.__name__ != 'PE_PE':
-        pe_pe = xtuml.navigate_one(pe_pe).PE_PE[8001]()
+        pe_pe = one(pe_pe).PE_PE[8001]()
     
-    
-    ep_pkg = xtuml.navigate_one(pe_pe).EP_PKG[8000]()
+    ep_pkg = one(pe_pe).EP_PKG[8000]()
     if ep_pkg:
         return get_defining_component(ep_pkg)
     
-    return xtuml.navigate_one(pe_pe).C_C[8003]()
+    return one(pe_pe).C_C[8003]()
+
+
+def get_parent_label(pe_pe):
+    '''
+    Get a human readable label for the parent in which pe_pe is defined.
+    '''
+    if pe_pe is None:
+        return ''
+
+    if pe_pe.__class__.__name__ != 'PE_PE':
+        pe_pe = one(pe_pe).PE_PE[8001]()
     
+    inst = one(pe_pe).EP_PKG[8000]() or one(pe_pe).C_C[8003]()
+    if inst is None:
+        return ''
+    
+    label = get_parent_label(inst)
+    if label:
+        label += '::'
+
+    return label + inst.Name
+
 
 class Scope(object):
 
@@ -203,8 +223,7 @@ class ActionPrebuilder(xtuml.tools.Walker):
         return one(self.pe_pe_cache).S_EE[8001](where(Key_Lett=key_letter))
     
     def r_rel(self, rel_id):
-        where_clause = lambda sel: 'r%d' % sel.Numb == rel_id.lower()
-        return self.any('R_REL', where_clause)
+        return one(self.pe_pe_cache).R_REL[8001](where(Numb=int(rel_id[1:])))
     
     def cnst_syc(self, name):
         where_clause = lambda sel:(ooaofooa.is_contained_in(sel, self.c_c) or
@@ -1097,18 +1116,34 @@ class ActionPrebuilder(xtuml.tools.Walker):
         
         return v_val
     
-    def accept_EnumNode(self, node):
+    def accept_EnumOrNamedConstantNode(self, node):
         s_dt = self.s_dt(node.namespace)
         while one(s_dt).S_UDT[17]():
             s_dt = one(s_dt).S_UDT[17].S_DT[18]()
-            
+
         s_enum = one(s_dt).S_EDT[17].S_ENUM[27](where(name=node.name))
-        v_val = self.v_val(node)
-        v_len = self.new('V_LEN')
+        cnst_csp = self.any('CNST_CSP', where(InformalGroupName=node.namespace))
+        cnst_syc = one(cnst_csp).CNST_SYC[1504](where(Name=node.name))
+            
+        if s_enum:
+            v_val = self.v_val(node)
+            v_len = self.new('V_LEN')
         
-        relate(v_val, s_dt, 820)
-        relate(v_len, v_val, 801)
-        relate(v_len, s_enum, 824)
+            relate(v_val, s_dt, 820)
+            relate(v_len, v_val, 801)
+            relate(v_len, s_enum, 824)
+
+        elif cnst_syc:
+            s_dt = one(cnst_syc).S_DT[1500]()
+            v_val = self.v_val(node)
+            v_scv = self.new('V_SCV')
+            
+            relate(v_val, v_scv, 801)
+            relate(v_val, s_dt, 820)
+            relate(v_scv, cnst_syc, 850)
+
+        else:
+            raise Exception("Unknown identifier '%s::%s'" % (node.namespace, node.name))
         
         return v_val
     
@@ -1464,17 +1499,17 @@ class ActionPrebuilder(xtuml.tools.Walker):
         return act_smt
     
 class BridgePrebuilder(ActionPrebuilder):
+    element_type = 'Bridge'
+    label = None
+    
     def __init__(self, metamodel, s_brg):
-        self._s_brg = s_brg
         s_ee = one(s_brg).S_EE[19]()
         c_c = get_defining_component(s_ee)
-        ActionPrebuilder.__init__(self, metamodel, c_c)      
+        self._s_brg = s_brg
+        self.label = '::'.join([get_parent_label(s_ee), s_ee.Name, s_brg.Name])
+
+        ActionPrebuilder.__init__(self, metamodel, c_c)
 
-    @property
-    def label(self):
-        s_ee = one(self._s_brg).S_EE[19]()
-        return '%s::%s' % (s_ee.Name, self._s_brg.Name)
-    
     def accept_BodyNode(self, node):
         act_brb = self.new('ACT_BRB')
         relate(act_brb, self._s_brg, 697)
@@ -1498,15 +1533,16 @@ class BridgePrebuilder(ActionPrebuilder):
 
 
 class FunctionPrebuilder(ActionPrebuilder):
+    element_type = 'Function'
+    label = None
+    
     def __init__(self, metamodel, s_sync):
         self._s_sync = s_sync
         c_c = get_defining_component(s_sync)
+        self.label = get_parent_label(s_sync) + '::' + s_sync.Name
+        
         ActionPrebuilder.__init__(self, metamodel, c_c)      
-    
-    @property
-    def label(self):
-        return self._s_sync.Name
-    
+
     def accept_BodyNode(self, node):
         act_fnb = self.new('ACT_FNB')
         relate(act_fnb, self._s_sync, 695)
@@ -1533,11 +1569,15 @@ class FunctionPrebuilder(ActionPrebuilder):
 
 
 class OperationPrebuilder(ActionPrebuilder):
+    element_type = 'Operation'
+    label = None
     
     def __init__(self, metamodel, o_tfr):
         self._o_tfr = o_tfr
         self._o_obj = one(o_tfr).O_OBJ[115]()
         c_c = get_defining_component(self._o_obj)
+        self.label = '::'.join([get_parent_label(self._o_obj), self._o_obj.Name,
+                                o_tfr.Name])
         ActionPrebuilder.__init__(self, metamodel, c_c)      
 
     def find_symbol(self, node, name):
@@ -1548,10 +1588,6 @@ class OperationPrebuilder(ActionPrebuilder):
         
         return v_var
     
-    @property
-    def label(self):
-        return '%s::%s' % (self._o_obj.Name, self._o_tfr.Name)
-    
     def accept_BodyNode(self, node):
         act_opb = self.new('ACT_OPB')
         relate(act_opb, self._o_tfr, 696)
@@ -1578,11 +1614,28 @@ class OperationPrebuilder(ActionPrebuilder):
 
 
 class TransitionPrebuilder(ActionPrebuilder):
+    element_type = None
+    label = None
     
     def __init__(self, metamodel, sm_act):
         self._sm_act = sm_act
+        self._sm_evt = (one(sm_act).SM_AH[514].SM_TAH[513].SM_TXN[530].
+                        SM_NSTXN[507].SM_SEME[504].SM_SEVT[503].SM_EVT[525]())
+        self._sm_state = one(sm_act).SM_AH[514].SM_MOAH[513].SM_STATE[511]()
         self._o_obj = (one(sm_act).SM_SM[515].SM_ISM[517].O_OBJ[518]() or
                        one(sm_act).SM_SM[515].SM_ASM[517].O_OBJ[519]())
+        
+        if self._sm_evt:
+            self.element_type = 'Transition'
+            action_name = self.get_event_name(self._sm_evt)
+            
+        elif self._sm_state:
+            self.element_type = 'State'
+            action_name = self._sm_state.Name
+            
+        self.label = '::'.join([get_parent_label(self._o_obj), self._o_obj.Name,
+                                action_name])
+
         c_c = get_defining_component(self._o_obj)
         ActionPrebuilder.__init__(self, metamodel, c_c)
 
@@ -1593,11 +1646,28 @@ class TransitionPrebuilder(ActionPrebuilder):
             v_var = one(v_int).V_VAR[814]()
         
         return v_var
-    
-    @property
-    def label(self):
-        return ''
+
+    def get_event_name(self, event):
+        sm_nlevt = one(event).SM_SEVT[525].SM_NLEVT[526]()
+        sm_sgevt = one(event).SM_SEVT[525].SM_SGEVT[526]()
+        sm_pevt = one(sm_nlevt).SM_PEVT[527]()
+
+        # If polymorphic and the polymorphic event is not local,
+        # use the poly local class name
+        if sm_pevt and sm_pevt.SM_ID != sm_nlevt.SM_ID:
+            return event.Mning + "::" + sm_pevt.localClassName
         
+        # If an orphaned polymorphic, append that to the name
+        if sm_nlevt is not None and sm_pevt is None:
+            return event.Mning + "::Orphaned"
+
+        #  If a signal event we use only the derived label
+        if sm_sgevt is not None:
+            return event.Drv_Lbl
+
+        # otherwise we combine the label with the event Mning
+        return event.Drv_Lbl + ": " + event.Mning
+
     def accept_BodyNode(self, node):
         sm_moah = one(self._sm_act).SM_AH[514].SM_MOAH[513]()
         if sm_moah is not None:
@@ -1648,6 +1718,7 @@ class TransitionPrebuilder(ActionPrebuilder):
 
     
 class DerivedAttributePrebuilder(ActionPrebuilder):
+    element_type = 'Derived Base Attribute'
     
     def __init__(self, metamodel, o_dbattr):
         self._o_dbattr = o_dbattr
@@ -1666,8 +1737,8 @@ class DerivedAttributePrebuilder(ActionPrebuilder):
     @property
     def label(self):
         o_attr = one(self._o_dbattr).O_BATTR[107].O_ATTR[106]()
-        return '%s::%s' % (self._o_obj.Name, o_attr.Name)
-    
+        return '::'.join([get_parent_label(self._o_obj), self._o_obj.Name, o_attr.Name])
+
     def accept_BodyNode(self, node):
         act_dab = self.new('ACT_DAB')
         relate(act_dab, self._o_dbattr, 693)
@@ -1682,19 +1753,18 @@ class DerivedAttributePrebuilder(ActionPrebuilder):
     
     
 class RequiredOperationPrebuilder(ActionPrebuilder):
+    element_type = 'Required Operaton'
+    label = None
     
     def __init__(self, metamodel, spr_ro):
         self._spr_ro = spr_ro
         c_c = one(spr_ro).SPR_REP[4502].C_R[4500].C_IR[4009].C_PO[4016].C_C[4010]()
-        ActionPrebuilder.__init__(self, metamodel, c_c)      
+        c_i = one(spr_ro).SPR_REP[4502].C_R[4500].C_IR[4009].C_I[4012]()
+        c_po = one(spr_ro).SPR_REP[4502].C_R[4500].C_IR[4009].C_PO[4016]()
+        self.label = '::'.join([get_parent_label(c_c), c_c.Name, c_po.Name,
+                                c_i.Name, spr_ro.Name])
+        ActionPrebuilder.__init__(self, metamodel, c_c)
 
-    @property
-    def label(self):
-        # interface = one(self._spr_ro).SPR_REP[4502].C_R[4500].C_IR[4009].C_I[4012]()
-        # port = one(self._spr_ro).SPR_REP[4502].C_R[4500].C_IR[4009].C_PO[4016]()
-        # return '%s::%s::%s' % (port.Name, interface.Name, self._spr_ro.Name)
-        return ''
-    
     def accept_BodyNode(self, node):
         act_rob = self.new('ACT_ROB')
         relate(act_rob, self._spr_ro, 685)
@@ -1718,16 +1788,18 @@ class RequiredOperationPrebuilder(ActionPrebuilder):
 
 
 class RequiredSignalPrebuilder(ActionPrebuilder):
+    element_type = 'Required Signal'
+    label = None
     
     def __init__(self, metamodel, spr_rs):
         self._spr_rs = spr_rs
         c_c = one(spr_rs).SPR_REP[4502].C_R[4500].C_IR[4009].C_PO[4016].C_C[4010]()
+        c_i = one(spr_rs).SPR_REP[4502].C_R[4500].C_IR[4009].C_I[4012]()
+        c_po = one(spr_rs).SPR_REP[4502].C_R[4500].C_IR[4009].C_PO[4016]()
+        self.label = '::'.join([get_parent_label(c_c), c_c.Name, c_po.Name,
+                                c_i.Name, spr_rs.Name])
         ActionPrebuilder.__init__(self, metamodel, c_c)  
 
-    @property
-    def label(self):
-        return ''
-    
     def accept_BodyNode(self, node):
         act_rsb = self.new('ACT_RSB')
         relate(act_rsb, self._spr_rs, 684)
@@ -1751,19 +1823,17 @@ class RequiredSignalPrebuilder(ActionPrebuilder):
 
 
 class ProvidedOperationPrebuilder(ActionPrebuilder):
-
+    element_type = 'Provided Operation'
+    
     def __init__(self, metamodel, spr_po):
         self._spr_po = spr_po
         c_c = one(spr_po).SPR_PEP[4503].C_P[4501].C_IR[4009].C_PO[4016].C_C[4010]()
+        c_i = one(spr_po).SPR_PEP[4503].C_P[4501].C_IR[4009].C_I[4012]()
+        c_po = one(spr_po).SPR_PEP[4503].C_P[4501].C_IR[4009].C_PO[4016]()
+        self.label = '::'.join([get_parent_label(c_c), c_c.Name, c_po.Name,
+                                c_i.Name, spr_po.Name])
         ActionPrebuilder.__init__(self, metamodel, c_c)  
         
-    @property
-    def label(self):
-        # interface = one(self._spr_po).SPR_PEP[4503].C_P[4501].C_IR[4009].C_I[4012]()
-        # port = one(self._spr_po).SPR_PEP[4503].C_P[4501].C_IR[4009].C_PO[4016]()
-        # return '%s::%s::%s' % (port.Name, interface.Name, self._spr_po.Name)
-        return ''
-    
     def accept_BodyNode(self, node):
         act_pob = self.new('ACT_POB')
         relate(act_pob, self._spr_po, 687)
@@ -1787,16 +1857,19 @@ class ProvidedOperationPrebuilder(ActionPrebuilder):
 
 
 class ProvidedSignalPrebuilder(ActionPrebuilder):
-
+    element_type = 'Provided Signal'
+    label = None
+    
     def __init__(self, metamodel, spr_ps):
         self._spr_ps = spr_ps
         c_c = one(spr_ps).SPR_PEP[4503].C_P[4501].C_IR[4009].C_PO[4016].C_C[4010]()
+        c_i = one(spr_ps).SPR_PEP[4503].C_P[4501].C_IR[4009].C_I[4012]()
+        c_po = one(spr_ps).SPR_PEP[4503].C_P[4501].C_IR[4009].C_PO[4016]()
+        self.label = '::'.join([get_parent_label(c_c), c_c.Name, c_po.Name,
+                                c_i.Name, spr_ps.Name])
+
         ActionPrebuilder.__init__(self, metamodel, c_c)  
 
-    @property
-    def label(self):
-        return ''
-    
     def accept_BodyNode(self, node):
         act_psb = self.new('ACT_PSB')
         relate(act_psb, self._spr_ps, 686)
@@ -1847,8 +1920,10 @@ def prebuild_action(instance):
         'SPR_PS': ProvidedSignalPrebuilder
     }
     metaclass = xtuml.get_metaclass(instance)
-    walker = walker_map[metaclass.kind](metaclass.metamodel, instance)
-    logger.info('processing action %s' % walker.label)
+    walker = walker_map[metaclass.kind](
+        metaclass.metamodel, instance)
+    logger.info('processing %s at %s' %
+                (walker.element_type, walker.label))
     # walker.visitors.append(xtuml.tools.NodePrintVisitor())
     root = oal.parse(instance.Action_Semantics_internal)
     return walker.accept(root)
